@@ -1,5 +1,3 @@
-from mathics.core.definitions import Definitions
-from mathics.core.evaluation import Evaluation
 from mathics.session import MathicsSession
 
 session = MathicsSession(add_builtin=True, catch_interrupt=False)
